@@ -6,8 +6,8 @@ pyFiles = glob.glob("lib/doit/*.py") + glob.glob("tests/*.py")
 
 def task_checker():
     for file in pyFiles:
-        yield {'action': "pychecker %s"% file, 
-               'name':file, 
+        yield {'action': "pyflakes %s"% file,
+               'name':file,
                'dependencies':(file,)}
 
 def task_nose():
