@@ -19,27 +19,29 @@ class TaskError(Exception):
     """Error while trying to execute task."""
     pass
 
-# interface 
+# interface
 class BaseTask(object):
     """Base class for all tasks objects
 
     @cvar CAPTURE_OUT: (bool) stdout from the task to be captured
     @cvar CAPTURE_ERR: (bool) stderr from the task to be captured
-    @ivar name string 
+    @ivar name string
     @ivar action see derived classes
     @ivar dependencies list of all dependencies
     @ivar targets list of targets
-    @ivar folder_dep: (list) 
-    @ivar task_dep: (list)
-    @ivar file_dep: (list)
+    @ivar folder_dep: (list - string)
+    @ivar task_dep: (list - string)
+    @ivar file_dep: (list - string)
     @ivar run_once: (bool) task without dependencies should run
+    @ivar is_subtask: (bool) indicate this task is a subtask.
     """
 
     CAPTURE_OUT = False
     CAPTURE_ERR = False
 
-    def __init__(self,name,action,dependencies=(),targets=()):
-        """Init."""    
+    def __init__(self,name,action,dependencies=(),targets=(),setup=None,
+                 is_subtask=False):
+        """Init."""
         # dependencies parameter must be a list
         if not ((isinstance(dependencies,list)) or
                 (isinstance(dependencies,tuple))):
@@ -57,11 +59,13 @@ class BaseTask(object):
         self.action = action
         self.dependencies = dependencies
         self.targets = targets
-        self.run_once = False        
+        self.setup = setup
+        self.run_once = False
+        self.is_subtask = is_subtask
 
         # there are 3 kinds of dependencies: file, task, and folder
-        self.folder_dep = [] 
-        self.task_dep = [] 
+        self.folder_dep = []
+        self.task_dep = []
         self.file_dep = []
         for dep in self.dependencies:
             # True on the list. set run_once
@@ -75,9 +79,9 @@ class BaseTask(object):
             elif dep.endswith('/'):
                 self.folder_dep.append(dep)
             # task dep starts with a ':'
-            elif dep.startswith(':'):                    
+            elif dep.startswith(':'):
                 self.task_dep.append(dep[1:])
-            # file dep 
+            # file dep
             elif isinstance(dep,str):
                 self.file_dep.append(dep)
 
@@ -87,13 +91,14 @@ class BaseTask(object):
                    "at the same time. (just remove True)")
             raise InvalidTask(msg % name)
 
+
     def execute(self):
         """Executes the task.
 
-        @raise TaskFailed: 
+        @raise TaskFailed:
         @raise TaskError:
         """
-        raise InvalidTask("Not Implemented")
+        raise Exception("Not Implemented")
 
 
     def title(self):
@@ -106,12 +111,17 @@ class BaseTask(object):
 
 class CmdTask(BaseTask):
     """Command line task. Spawns a new process."""
- 
-    def __init__(self,name,action,dependencies=(),targets=()):
+
+    def __init__(self, name, action, dependencies=(), targets=(), setup=None,
+                 is_subtask=False):
         """Init."""
-        assert isinstance(action,str),\
-            "'action' from CmdTask must be a string."
-        BaseTask.__init__(self,name,action,dependencies,targets)
+        assert isinstance(action,str) or isinstance(action, list),\
+            "'action' from CmdTask must be a string or list."
+
+        if isinstance(action, str):
+            action = [action]
+        BaseTask.__init__(self, name, action, dependencies, targets, setup,
+                          is_subtask)
 
     def execute(self):
         # set Popen stream parameters
@@ -124,35 +134,36 @@ class CmdTask(BaseTask):
         else:
             stderr = subprocess.PIPE
 
-        # spawn task process
-        process = subprocess.Popen(self.action,stdout=stdout,
-                                 stderr=stderr, shell=True)
+        for action in self.action:
+            # spawn task process
+            process = subprocess.Popen(action,stdout=stdout,
+                                     stderr=stderr, shell=True)
 
-        # log captured stream
-        out,err = process.communicate()
-        if out:
-            logger.log('stdout',out)
-        if err:
-            logger.log('stderr',err)
+            # log captured stream
+            out,err = process.communicate()
+            if out:
+                logger.log('stdout',out)
+            if err:
+                logger.log('stderr',err)
 
-        # task error - based on: 
-        # http://www.gnu.org/software/bash/manual/bashref.html#Exit-Status
-        # it doesnt make so much difference to return as Error or Failed anyway
-        if process.returncode > 125:
-            raise TaskError("Command error: '%s' returned %s" % 
-                            (self.action,process.returncode))
+            # task error - based on:
+            # http://www.gnu.org/software/bash/manual/bashref.html#Exit-Status
+            # it doesnt make so much difference to return as Error or Failed anyway
+            if process.returncode > 125:
+                raise TaskError("Command error: '%s' returned %s" %
+                                (self.action,process.returncode))
+
+            # task failure
+            if process.returncode != 0:
+                raise TaskFailed("Command failed: '%s' returned %s" %
+                                 (self.action,process.returncode))
 
-        # task failure
-        if process.returncode != 0:
-            raise TaskFailed("Command failed: '%s' returned %s" % 
-                             (self.action,process.returncode))
 
-            
     def __str__(self):
-        return "Cmd: %s"% self.action
+        return "\n\t".join(["Cmd: %s"% action for action in self.action])
 
     def __repr__(self):
-        return "<CmdTask: %s - '%s'>"% (self.name,self.action)
+        return "<CmdTask: %s - '%s'>"% (self.name, "\n\t".join(self.action))
 
 
 
@@ -164,10 +175,12 @@ class PythonTask(BaseTask):
     @ivar kwargs: (dict) dict to be passed to the callable
     """
 
-    def __init__(self,name,action,dependencies=(),targets=(),args=None,kwargs=None):
+    def __init__(self, name, action, dependencies=(), targets=(), setup=None,
+                 is_subtask=False, args=None, kwargs=None):
         """Init."""
         assert callable(action),"'action' from PythonTask must be a 'callable'."
-        BaseTask.__init__(self,name,action,dependencies,targets)
+        BaseTask.__init__(self, name, action, dependencies, targets, setup,
+                          is_subtask)
         if args is None:
             self.args = []
         else:
@@ -179,7 +192,7 @@ class PythonTask(BaseTask):
             self.kwargs = kwargs
 
     def execute(self):
-        # set std stream 
+        # set std stream
         if self.CAPTURE_OUT:
             old_stdout = sys.stdout
             sys.stdout = StringIO.StringIO()
@@ -193,6 +206,11 @@ class PythonTask(BaseTask):
             # Python2.4
             try:
                 result = self.action(*self.args,**self.kwargs)
+            # in python 2.4 SystemExit and KeyboardInterrupt subclass
+            # from Exception.
+            except (SystemExit, KeyboardInterrupt), exp:
+                raise
+
             except Exception, exception:
                 error = TaskError(exception)
                 error.originalException = traceback.format_exception(\
@@ -214,7 +232,7 @@ class PythonTask(BaseTask):
         if not result:
             raise TaskFailed("Python Task failed: '%s' returned %s" %
                              (self.action, result))
-                    
+
     def __str__(self):
         # get object description excluding runtime memory address
         return "Python: %s"% str(self.action)[1:].split(' at ')[0]
@@ -236,3 +254,61 @@ class GroupTask(BaseTask):
 
     def __repr__(self):
         return "<GroupTask: %s>"% self.name
+
+
+
+def create_task(name,action,dependencies,targets,setup,*args,**kwargs):
+    """ create a BaseTask acording to action type
+
+    @param name: (string) task name
+    @param action: value dependes on the type of the task
+    @param dependencies: (list of strings) each item is a file path or
+    another task (prefixed with ':')
+    @param targets: (list of strings) items are file paths.
+    @param args: optional positional arguments for task.
+    @param kwargs: optional keyword arguments for task.
+    """
+    # a string.
+    if isinstance(action,str) or isinstance(action,list):
+        return CmdTask(name,action,dependencies,targets,setup)
+    # a callable.
+    elif callable(action):
+        return PythonTask(name,action,dependencies,targets,setup,*args,**kwargs)
+    elif action is None:
+        return GroupTask(name,action,dependencies,targets,setup)
+    else:
+        raise InvalidTask("Invalid task type. %s:%s" % (name,action.__class__))
+
+def dict_to_task(task_dict):
+    """Create a task instance from dictionary.
+
+    The dictionary has the same format as returned by task-generators
+    from dodo files.
+
+    @param task_dict: (dict) task representation as a dict.
+    @raise L{InvalidTask}:
+    """
+    # TASK_ATTRS: sequence of know attributes(keys) of a task dict.
+    TASK_ATTRS = ('name','action','dependencies','targets','args','kwargs','setup')
+    # FIXME check field 'name'
+
+    # check required fields
+    if 'action' not in task_dict:
+        raise InvalidTask("Task %s must contain field action. %s"%
+                          (task_dict['name'],task_dict))
+
+    # user friendly. dont go ahead with invalid input.
+    for key in task_dict.keys():
+        if key not in TASK_ATTRS:
+            raise InvalidTask("Task %s contain invalid field: %s"%
+                              (task_dict['name'],key))
+
+    return create_task(task_dict.get('name'),
+                       task_dict.get('action'),
+                       task_dict.get('dependencies',[]),
+                       task_dict.get('targets',[]),
+                       task_dict.get('setup',None),
+                       args=task_dict.get('args',[]),
+                       kwargs=task_dict.get('kwargs',{}))
+
+
