@@ -1,4 +1,4 @@
-import os 
+import os
 
 from doit.dependency import Dependency
 
@@ -23,14 +23,17 @@ def get_abspath(relativePath):
 # taskId_dependency => signature(dependency)
 # taskId is md5(CmdTask.task)
 
-TESTDBM = "testdbm"
+TESTDB = "testdb"
+
 class DependencyTestBase(object):
     def setUp(self):
-        self.d = Dependency(TESTDBM)
+        if os.path.exists(TESTDB):
+            os.remove(TESTDB)
+        self.d = Dependency(TESTDB)
 
     def tearDown(self):
-        if os.path.exists(TESTDBM):
-            os.remove(TESTDBM)
+        if not self.d._closed:
+            self.d.close()
 
 class TestDependencyDb(DependencyTestBase):
 
@@ -40,26 +43,26 @@ class TestDependencyDb(DependencyTestBase):
         value = self.d._get("taskId_X","dependency_A")
         assert "da_md5" == value, value
 
-    # 
+    #
     def test_setPersistence(self):
         # save and close db
         self.d._set("taskId_X","dependency_A","da_md5")
-        del self.d
+        self.d.close()
 
         # open it again and check the value
-        d2 = Dependency(TESTDBM)
+        d2 = Dependency(TESTDB)
         value = d2._get("taskId_X","dependency_A")
         assert "da_md5" == value, value
-        
+
     def test_new(self):
         # save and close db
         self.d._set("taskId_X","dependency_A","da_md5")
-        del self.d
+        self.d.close()
 
         # open same file but with new parameter
-        d2 = Dependency(TESTDBM, new=True)
+        d2 = Dependency(TESTDB, new=True)
         assert 0 == len(d2._db)
-        
+
 
     # _get must return None if entry doesnt exist.
     def test_getNonExistent(self):
@@ -78,18 +81,37 @@ class TestDependencyDb(DependencyTestBase):
         expected = "a1bb792202ce163b4f0d17cb264c04e1"
         value = self.d._get("taskId_X",filePath)
         assert expected == value, value
-        
+
+    def test_remove(self):
+        self.d._set("taskId_ZZZ","dep_1","12")
+        self.d._set("taskId_ZZZ","dep_2","13")
+        self.d._set("taskId_YYY","dep_1","14")
+        self.d.remove("taskId_ZZZ")
+        assert None == self.d._get("taskId_ZZZ","dep_1")
+        assert None == self.d._get("taskId_ZZZ","dep_2")
+        assert "14" == self.d._get("taskId_YYY","dep_1")
+
+    def test_remove_all(self):
+        self.d._set("taskId_ZZZ","dep_1","12")
+        self.d._set("taskId_ZZZ","dep_2","13")
+        self.d._set("taskId_YYY","dep_1","14")
+        self.d.remove_all()
+        assert None == self.d._get("taskId_ZZZ","dep_1")
+        assert None == self.d._get("taskId_ZZZ","dep_2")
+        assert None == self.d._get("taskId_YYY","dep_1")
+
+
     def test_notModified(self):
         # create a test dependency file
         filePath = get_abspath("data/dependency1")
         ff = open(filePath,"w")
         ff.write("i am the first dependency ever for doit")
-        ff.close()      
+        ff.close()
         # save it
         self.d.save("taskId_X",filePath)
-        
+
         assert not self.d.modified("taskId_X",filePath)
-        
+
     def test_yesModified(self):
         # create a test dependency file
         filePath = get_abspath("data/dependency1")
@@ -102,7 +124,7 @@ class TestDependencyDb(DependencyTestBase):
         ff = open(filePath,"a")
         ff.write(" - with the first modification!")
         ff.close()
-        
+
         assert self.d.modified("taskId_X",filePath)
 
     # if there is no entry for dependency. it is like modified.
@@ -117,9 +139,9 @@ class TestDependencyDb(DependencyTestBase):
 
 
 class TestTaskDependency(DependencyTestBase):
-        
+
     # whenever a task has a dependency the runner checks if this dependency
-    # was modified since last successful run. if not the task is skipped.    
+    # was modified since last successful run. if not the task is skipped.
 
     def test_saveDependencies(self):
         filePath = get_abspath("data/dependency1")
@@ -166,7 +188,7 @@ class TestTaskDependency(DependencyTestBase):
         ff = open(filePath,"a")
         ff.write(" part2")
         ff.close()
-        
+
         # execute again
         assert not self.d.up_to_date(taskId,dependencies,[],False)
 
@@ -193,20 +215,21 @@ class TestTaskDependency(DependencyTestBase):
         dependencies = [get_abspath("data/dependency1")]
         targets = [filePath]
         self.d.save_dependencies(taskId,dependencies)
-        # first time execute
-        assert not self.d.up_to_date(taskId,dependencies,targets,False)
-        self.d.save_dependencies(taskId,targets)
-        # second time no
+        # up-to-date because target exist
         assert self.d.up_to_date(taskId,dependencies,targets,False)
 
-        # a small change on the file
-        ff = open(filePath,"a")
-        ff.write(" part2")
-        ff.close()
-        
-        # execute again
-        assert not self.d.up_to_date(taskId,dependencies,targets,False)
-
+    def test_upToDate_targetFolder(self):
+        # folder not there. task is not up-to-date
+        dependencies = [get_abspath("data/dependency1")]
+        taskId = "task x"
+        self.d.save_dependencies(taskId,dependencies)
+        folderPath = get_abspath("data/target-folder")
+        if os.path.exists(folderPath):
+            os.rmdir(folderPath)
+        assert not self.d.up_to_date(taskId,dependencies,[folderPath],False)
+        # create folder. task is up-to-date
+        os.mkdir(folderPath)
+        assert self.d.up_to_date(taskId,dependencies,[folderPath],False)
 
 class TestRunOnceDependency(DependencyTestBase):
 
@@ -215,4 +238,4 @@ class TestRunOnceDependency(DependencyTestBase):
         assert not self.d.up_to_date(taskId,[],[],True)
         self.d.save_run_once(taskId)
         assert self.d.up_to_date(taskId,[],[],True)
-        
+
